# -*- coding: utf-8 -*-#
#!/usr/bin/env python
# ------------------------------------------------------------
# mediaLocker.py
#
# Authors:
# Mike Driscoll, mike@pythonlibrary.org
# Werner F Bruhin, werner.bruhin@free.fr
#
# This program demonstrates how to work with a database
# using wxPython and SqlAlchemy.

# It is also an example of MVC concepts and how to put
# together a fully working wxPython application.
# ------------------------------------------------------------
import logging
format = '%(name)s - %(levelname)s - %(filename)s - %(lineno)d - %(message)s'
logging.basicConfig(format=format, level=logging.DEBUG)

import functools

# import this first for wx version requirement
import base_app as ba
import wx
import wx.lib.agw.aui as aui
from wx.lib.wordwrap import wordwrap

# needs to be changed to fully support i18n
_ = wx.GetTranslation

import controllers.p_books as pBooks

import controllers.d_publishers as dlgPublishers
import controllers.d_persons as dlgPersons

import libui.uiutils as uiutils

########################################################################
class MediaLocker(wx.Frame):
    """The MediaLocker application controller"""

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        wx.Frame.__init__(self, None, title="MvP Media Organizer",
                          size=(800, 600))
        self._mgr = aui.AuiManager(self)
        panel = pBooks.Books(self)
        self._mgr.AddPane(panel.view, aui.AuiPaneInfo().
                          Name("paneBooks").Caption("Books pane").Centre().
                          CaptionVisible(False).
                          CloseButton(False).
                          MaximizeButton(False).
                          MinimizeButton(False))
                
        tb = self.doMainTB(self._mgr, self)
        self.doMainMenu()
        
        self._mgr.Update()

        self.Show()
        
    def doAboutBox(self, evt):
        """
        Create and show an about box
        """
        info = wx.AboutDialogInfo()
        info.Name = "MediaLocker"
        info.Version = "0.2"  # TODO: pull from somewhere?
        
        info.Description = wordwrap("MediaLocker is a wxPython application to help you"
                                    " keep track of your stuff."
                                    
                                    "\n\nThis program also demonstrates how to work with a database "
                                    "using wxPython and SqlAlchemy."
                                    
                                    "\n\nIt is also an example of MVC concepts and how to put"
                                    " together a fully working wxPython application.",
                                    350, wx.ClientDC(self))
        info.WebSite = ("https://bitbucket.org/driscollis/medialocker", "Download the Source")
        info.Developers = ["Mike Driscoll",
                           "Werner F Bruhin"]
        wx.AboutBox(info)
        
    def doMainMenu(self):
        """
        Create the main menu
        """
        menubar = wx.MenuBar()
        
        # file menu
        fileMenu = wx.Menu()
        closeMenuItem = fileMenu.Append(wx.NewId(), "Close",
                                        "Close the application")
        self.Bind(wx.EVT_MENU, self.onClose, closeMenuItem)
        menubar.Append(fileMenu, "&File")
        
        # manage menu
        manageMenu = wx.Menu()
        managePersonMenuItem = manageMenu.Append(wx.NewId(), "Person",
                                                 "Manage people")
        self.Bind(wx.EVT_MENU, self.doPersonDlg, managePersonMenuItem)
        
        managePubliserMenuItem = manageMenu.Append(wx.NewId(), "Add Publisher",
                                                   "Manage publishers")
        self.Bind(wx.EVT_MENU, self.doPublisherDlg, managePubliserMenuItem)
        menubar.Append(manageMenu, "&Manage")
        
        # help menu
        helpMenu = wx.Menu()
        aboutMenuItem = helpMenu.Append(wx.NewId(), "&About",
                                        "About the program")
        self.Bind(wx.EVT_MENU, self.doAboutBox, aboutMenuItem)
        menubar.Append(helpMenu, "&Help")
        
        self.SetMenuBar(menubar)


    def doMainTB(self, mgr, owner):
        mainTB = aui.AuiToolBar(owner, wx.ID_ANY, 
                            wx.DefaultPosition, wx.DefaultSize,
                            agwStyle=aui.AUI_TB_OVERFLOW | aui.AUI_TB_HORZ_TEXT)
        mainTB.SetToolBitmapSize(wx.Size(16, 16))

        tools = [
         ['person', wx.NewId(), _("Person"), _("Maintain person"),
                              _("Maintain person information"),
                              self.doPersonDlg,
                              "person.png", aui.ITEM_NORMAL],

         ['publisher', wx.NewId(), _("Publisher"), _("Maintain publisher"),
                              _("Maintain publisher information"),
                              self.doPublisherDlg,
                              "publisher.png", aui.ITEM_NORMAL],

         ['contexthelp', wx.NewId(), _("Help"), _("Context help"),
                                _("Click on item for which you like to see help"),
                                self.onTBHelp,
                                "help.png", aui.ITEM_NORMAL]
        ]

        uiutils.doCreateToolbarItems(self, mainTB, tools)

        mainTB.Realize()
        mgr.AddPane(mainTB, aui.AuiPaneInfo().
                          Name("mainTB").Caption("Toolbar").
                          ToolbarPane().Top())            

    def onTBHelp(self, evt): #pylint: disable=W0613
        """Handle toolbar help event"""
        csHelp = wx.ContextHelp(self, True)
        
    def doPersonDlg(self, evt):
        dlg = dlgPersons.Persons(self).view
        try:
            dlg.ShowModal()
        finally:
            dlg.Destroy()
    
    def doPublisherDlg(self, evt):
        dlg = dlgPublishers.Publishers(self).view
        try:
            dlg.ShowModal()
        finally:
            dlg.Destroy()
            
    def onClose(self, evt):
        """
        Close the application
        """
        self.Close()

#----------------------------------------------------------------------
if __name__ == "__main__":
    app = ba.BaseApp(False)
    frame = MediaLocker()
    app.MainLoop()